/// <reference types="node" />
import { Credentials } from "../credentials/Credentials.js";
import { Entry } from "../core/Entry.js";
import { VaultSource } from "../core/VaultSource.js";
import { AttachmentDetails } from "../types.js";
export declare class AttachmentManager {
    /**
     * Get a new attachment ID
     * @static
     */
    static newAttachmentID(): string;
    _credentials: Credentials;
    _source: VaultSource;
    /**
     * Constructor for new attachment managers
     * @param vaultSource The vault source to attach to. This is
     *  normally set by the VaultSource automatically when unlocking a source.
     */
    constructor(vaultSource: VaultSource);
    /**
     * Get an attachment's data
     * @param entry Entry instance the attachment can be found on
     * @param attachmentID The attachment ID
     * @throws {Error} Throws if the attachment isn't found
     */
    getAttachment(entry: Entry, attachmentID: string): Promise<Buffer | ArrayBuffer>;
    /**
     * Get an attachment's details
     * @param entry Entry instance the attachment can be found on
     * @param attachmentID The attachment ID
     * @returns The details or null if the attachment isn't found
     */
    getAttachmentDetails(entry: Entry, attachmentID: string): Promise<AttachmentDetails>;
    /**
     * List all attachments
     * @param entry Entry instance the attachment can be found on
     */
    listAttachments(entry: Entry): Promise<Array<AttachmentDetails>>;
    /**
     * Remove an attachment, deleting the file and removing it from
     *  the entry
     * @param entry Entry instance the attachment can be found on
     * @param attachmentID The attachment ID
     */
    removeAttachment(entry: Entry, attachmentID: string): Promise<void>;
    /**
     * Write an attachment to an entry.
     * Creates a new attachment or updates an existing one, by attachment ID
     * @param entry Entry instance the attachment can be found on
     * @param attachmentID The attachment ID
     * @param attachmentData The attachment's data
     * @param name The file name
     * @param type The MIME type
     * @param timestamp Optional timestamp override for the creation of the
     *  attachment
     */
    setAttachment(entry: Entry, attachmentID: string, attachmentData: Buffer | ArrayBuffer, name: string, type: string, timestamp?: Date): Promise<void>;
    _checkAttachmentSupport(): void;
    _getAttachmentsCredentials(): Promise<Credentials>;
}
