import EventEmitter from "eventemitter3";
import { TextDatasource } from "./TextDatasource.js";
interface AuthHandlerFunction {
    (datasource: TextDatasource): void;
}
interface AuthHandlers {
    [type: string]: Array<AuthHandlerFunction>;
}
/**
 * Authentication manager
 * @augments EventEmitter
 * @memberof module:Buttercup
 */
export default class DatasourceAuthManager extends EventEmitter {
    /**
     * Get the shared DatasourceAuthManager instance
     * @returns The shared auth manager instance
     * @static
     * @memberof DatasourceAuthManager
     */
    static getSharedManager(): DatasourceAuthManager;
    _handlers: AuthHandlers;
    /**
     * Constructor for the auth manager
     */
    constructor();
    /**
     * Execute auth handlers for a datasource
     * @param datasourceType The type of datasource (slug)
     * @param datasourceInst The datasource instance
     * @returns A promise that resolves once execution has completed
     * @throws {Error} Throws if no handlers have been specified
     * @memberof DatasourceAuthManager
     */
    executeAuthHandlers(datasourceType: string, datasourceInst: TextDatasource): Promise<void>;
    /**
     * Register an auth handler
     * @param datasourceType The datasource type
     * @param handler The handler function
     * @example
     *  authManager.registerHandler("googledrive", datasource => {
     *      return renewTokens().then(({ accessToken, refreshToken }) => {
     *          datasource.token = accessToken;
     *          datasource.refreshToken = refreshToken;
     *      });
     *  });
     * @memberof DatasourceAuthManager
     * @throws {Error} Throws if the handler argument is not a function
     */
    registerHandler(datasourceType: string, handler: AuthHandlerFunction): void;
}
export {};
