import { WebDAVClient } from "webdav";
import { TextDatasource } from "./TextDatasource.js";
import { Credentials } from "../credentials/Credentials.js";
import { AttachmentDetails, BufferLike, DatasourceConfigurationWebDAV, DatasourceLoadedData, History, VaultID } from "../types.js";
/**
 * WebDAV datasource for reading and writing remote archives
 * @augments TextDatasource
 * @memberof module:Buttercup
 */
export default class WebDAVDatasource extends TextDatasource {
    protected _client: WebDAVClient;
    private _config;
    protected _path: string;
    /**
     * Constructor for the datasource
     * @param credentials Credentials for the datasource
     */
    constructor(credentials: Credentials);
    /**
     * The vault file's base directory
     * @memberof WebDAVDatasource
     */
    get baseDir(): any;
    /**
     * The WebDAV client instance
     * @memberof WebDAVDatasource
     */
    get client(): WebDAVClient;
    /**
     * The remote archive path
     * @memberof WebDAVDatasource
     */
    get path(): string;
    /**
     * Ensure attachment paths exist
     * @memberof WebDAVDatasource
     * @protected
     */
    _ensureAttachmentsPaths(vaultID: VaultID): Promise<void>;
    /**
     * Get encrypted attachment
     * - Loads the attachment contents from a file into a buffer
     * @param vaultID The ID of the vault
     * @param attachmentID The ID of the attachment
     * @memberof WebDAVDatasource
     */
    getAttachment(vaultID: VaultID, attachmentID: string): Promise<BufferLike>;
    /**
     * Get the datasource configuration
     * @memberof WebDAVDatasource
     */
    getConfiguration(): DatasourceConfigurationWebDAV;
    /**
     * Load archive history from the datasource
     * @param credentials The credentials for archive decryption
     * @returns A promise resolving archive history
     * @memberof WebDAVDatasource
     */
    load(credentials: Credentials): Promise<DatasourceLoadedData>;
    /**
     * Put attachment data
     * @param vaultID The ID of the vault
     * @param attachmentID The ID of the attachment
     * @param buffer The attachment data
     * @param details The attachment details
     * @memberof WebDAVDatasource
     */
    putAttachment(vaultID: VaultID, attachmentID: string, buffer: BufferLike, details: AttachmentDetails): Promise<void>;
    /**
     * Remove an attachment
     * @param vaultID The ID of the vault
     * @param attachmentID The ID of the attachment
     * @memberof WebDAVDatasource
     */
    removeAttachment(vaultID: VaultID, attachmentID: string): Promise<void>;
    /**
     * Save archive contents to the WebDAV service
     * @param history Archive history
     * @param credentials The credentials for encryption
     * @returns A promise resolving when the save is complete
     * @memberof WebDAVDatasource
     */
    save(history: History, credentials: Credentials): Promise<any>;
    /**
     * Whether or not the datasource supports attachments
     * @memberof WebDAVDatasource
     */
    supportsAttachments(): boolean;
    /**
     * Whether or not the datasource supports bypassing remote fetch operations
     * @returns True if content can be set to bypass fetch operations,
     *  false otherwise
     * @memberof WebDAVDatasource
     */
    supportsRemoteBypass(): boolean;
}
