import pathPosix from "path-posix";
import { TextDatasource } from "./TextDatasource.js";
import { fireInstantiationHandlers, registerDatasource } from "./register.js";
import { getSharedAppEnv } from "../env/appEnv.js";
import { getCredentials } from "../credentials/channel.js";
import { ATTACHMENT_EXT } from "../tools/attachments.js";
const MAX_DATA_SIZE = 200 * 1024 * 1024; // 200 MB
/**
 * WebDAV datasource for reading and writing remote archives
 * @augments TextDatasource
 * @memberof module:Buttercup
 */
export default class WebDAVDatasource extends TextDatasource {
    /**
     * Constructor for the datasource
     * @param credentials Credentials for the datasource
     */
    constructor(credentials) {
        super(credentials);
        const { data: credentialData } = getCredentials(credentials.id);
        const { datasource: datasourceConfig } = credentialData;
        const { endpoint, password, path, username } = (this._config = datasourceConfig);
        this._path = path;
        const createClient = getSharedAppEnv().getProperty("net/webdav/v1/newClient");
        if (typeof username === "string" && typeof password === "string") {
            this._client = createClient(endpoint, {
                username,
                password,
                maxBodyLength: MAX_DATA_SIZE,
                maxContentLength: MAX_DATA_SIZE
            });
        }
        else {
            this._client = createClient(endpoint, {
                maxBodyLength: MAX_DATA_SIZE,
                maxContentLength: MAX_DATA_SIZE
            });
        }
        this.type = "webdav";
        fireInstantiationHandlers("webdav", this);
    }
    /**
     * The vault file's base directory
     * @memberof WebDAVDatasource
     */
    get baseDir() {
        return pathPosix.dirname(this.path);
    }
    /**
     * The WebDAV client instance
     * @memberof WebDAVDatasource
     */
    get client() {
        return this._client;
    }
    /**
     * The remote archive path
     * @memberof WebDAVDatasource
     */
    get path() {
        return this._path;
    }
    /**
     * Ensure attachment paths exist
     * @memberof WebDAVDatasource
     * @protected
     */
    async _ensureAttachmentsPaths(vaultID) {
        const attachmentsDir = pathPosix.join(this.baseDir, ".buttercup", vaultID);
        await this.client.createDirectory(attachmentsDir, { recursive: true });
    }
    /**
     * Get encrypted attachment
     * - Loads the attachment contents from a file into a buffer
     * @param vaultID The ID of the vault
     * @param attachmentID The ID of the attachment
     * @memberof WebDAVDatasource
     */
    async getAttachment(vaultID, attachmentID) {
        await this._ensureAttachmentsPaths(vaultID);
        const attachmentPath = pathPosix.join(this.baseDir, ".buttercup", vaultID, `${attachmentID}.${ATTACHMENT_EXT}`);
        return this.client.getFileContents(attachmentPath);
    }
    /**
     * Get the datasource configuration
     * @memberof WebDAVDatasource
     */
    getConfiguration() {
        return this._config;
    }
    /**
     * Load archive history from the datasource
     * @param credentials The credentials for archive decryption
     * @returns A promise resolving archive history
     * @memberof WebDAVDatasource
     */
    load(credentials) {
        return this.hasContent
            ? super.load(credentials)
            : this.client.getFileContents(this.path, { format: "text" }).then((content) => {
                this.setContent(content);
                return super.load(credentials);
            });
    }
    /**
     * Put attachment data
     * @param vaultID The ID of the vault
     * @param attachmentID The ID of the attachment
     * @param buffer The attachment data
     * @param details The attachment details
     * @memberof WebDAVDatasource
     */
    async putAttachment(vaultID, attachmentID, buffer, details) {
        await this._ensureAttachmentsPaths(vaultID);
        const attachmentPath = pathPosix.join(this.baseDir, ".buttercup", vaultID, `${attachmentID}.${ATTACHMENT_EXT}`);
        await this.client.putFileContents(attachmentPath, buffer);
    }
    /**
     * Remove an attachment
     * @param vaultID The ID of the vault
     * @param attachmentID The ID of the attachment
     * @memberof WebDAVDatasource
     */
    async removeAttachment(vaultID, attachmentID) {
        await this._ensureAttachmentsPaths(vaultID);
        const attachmentPath = pathPosix.join(this.baseDir, ".buttercup", vaultID, `${attachmentID}.${ATTACHMENT_EXT}`);
        await this.client.deleteFile(attachmentPath);
    }
    /**
     * Save archive contents to the WebDAV service
     * @param history Archive history
     * @param credentials The credentials for encryption
     * @returns A promise resolving when the save is complete
     * @memberof WebDAVDatasource
     */
    async save(history, credentials) {
        const content = await super.save(history, credentials);
        await this.client.putFileContents(this.path, content);
    }
    /**
     * Whether or not the datasource supports attachments
     * @memberof WebDAVDatasource
     */
    supportsAttachments() {
        return true;
    }
    /**
     * Whether or not the datasource supports bypassing remote fetch operations
     * @returns True if content can be set to bypass fetch operations,
     *  false otherwise
     * @memberof WebDAVDatasource
     */
    supportsRemoteBypass() {
        return true;
    }
}
registerDatasource("webdav", WebDAVDatasource);
