import { VaultItem } from "./VaultItem.js";
import { EntryURLType } from "../tools/entry.js";
import { Group } from "./Group.js";
import { Vault } from "./Vault.js";
import { EntryChange, EntryPropertyValueType, EntryType, GroupID, PropertyKeyValueObject } from "../types.js";
/**
 * Entry class - some secret item, login or perhaps
 * even a credit card
 * @augments VaultItem
 */
export declare class Entry extends VaultItem {
    static Attributes: Readonly<{
        AttachmentPrefix: "BC_ENTRY_ATTACHMENT:";
        FacadeType: "BC_ENTRY_FACADE_TYPE";
        FieldTypePrefix: "BC_ENTRY_FIELD_TYPE:";
    }>;
    /**
     * Create a new Entry instance within a vault
     *  and group
     * @param vault The target vault instance
     * @param parentGroupID The target group to
     *  create the entry in (cannot be vault-level)
     * @static
     */
    static createNew(vault: Vault, parentGroupID: GroupID): Entry;
    /**
     * Delete the entry - either trashes the entry, or removes it completely.
     * If the entry is in the trash already, it is removed (including if there is no
     *    trash group). If the entry is in a normal group and a trash group exists, it
     *  is moved there instead of being deleted.
     * @param skipTrash Skip the trash and force-delete the entry
     * @see moveToGroup
     * @see Vault.getTrashGroup
     * @returns Whether or not the item was deleted
     */
    delete(skipTrash?: boolean): boolean;
    /**
     * Delete an attribute
     * @param attribute The attribute name
     * @throws {Error} Throws if the attribute doesn't exist, or cannot be deleted
     * @returns Self
     */
    deleteAttribute(attribute: string): this;
    /**
     * Delete a property
     * @throws {Error} Throws if property doesn't exist, or cannot be deleted
     * @param property The property to delete
     * @returns Self
     */
    deleteProperty(property: string): this;
    /**
     * Get an attribute
     * If no attribute name is specified, an object with all attributes and their
     * values is returned.
     * @param attribute The name of the attribute to fetch
     * @returns The attribute value or an object
     *  containing all attribute keys and their values if no attribute name
     *  is provided
     */
    getAttribute(attribute?: string): PropertyKeyValueObject | string | undefined;
    /**
     * Get an array of all history changes made to the entry
     */
    getChanges(): Array<EntryChange>;
    /**
     * Get the containing group for the entry
     * @returns The parent group
     * @throws {Error} Throws if no parent group found
     */
    getGroup(): Group;
    /**
     * Get a property value
     * If no property name is specified, an object with all properties and their
     * values is returned.
     * @param property The name of the property to fetch
     * @returns The property value or an object with all
     *  values if no property specified
     */
    getProperty(property?: string): PropertyKeyValueObject | string | undefined;
    /**
     * Set the value type of an entry property
     * @param property The property to get the value type of
     * @returns The property value type
     */
    getPropertyValueType(property: string): EntryPropertyValueType;
    /**
     * Get property values via RegExp expressions.
     * If no property expression is specified, it returns the empty behavior of
     * {@see Entry.getProperty}.
     * @param propertyExpression The expression to use to fetch properties. If
     *  a regular expression, it is then tested against all property keys. If a
     *  string, it's compared exactly.
     * @returns A key-value object of the matching properties
     */
    getProperties(propertyExpression?: RegExp | string): PropertyKeyValueObject;
    /**
     * Get the entry type
     * @returns
     */
    getType(): EntryType;
    /**
     * Get an array of URLs from the Entry
     * Returns an array of detected URL values in the Entry's properties. The
     * types of URLs can be configured by providing a preference:
     *  - "general" - General URLs (of any type, preferring "URL" named props)
     *  - "login" - Prefer URLs whose key has "login" in it
     *  - "icon" - Return only icon-like URLs
     *  - "any" - Return all found URLs
     * @param urlTypePreference The URL type preference
     * @returns An array of URLs
     */
    getURLs(urlTypePreference: EntryURLType): Array<string>;
    /**
     * Check if the entry is in the trash
     * @returns Whether or not the entry is in the trash
     */
    isInTrash(): boolean;
    /**
     * Move the entry to another group
     * @returns Returns self
     * @param group The target group
     */
    moveToGroup(group: Group): this;
    /**
     * Set an attribute on the entry
     * @param attribute The name of the attribute
     * @param value The value to set
     * @returns Returns self
     */
    setAttribute(attribute: string, value: string): this;
    /**
     * Set a property on the entry
     * @param property The property name
     * @param value The property value
     * @returns Returns self
     */
    setProperty(property: string, value: string): this;
    /**
     * Set the value type of an entry property
     * @param property The property to set the value type for
     * @param valueType The new value type to set, or `null` to remove
     * @returns Returns self
     */
    setPropertyValueType(property: string, valueType: EntryPropertyValueType | null): this;
    _updateRefs(): void;
}
