import { VaultItem } from "./VaultItem.js";
import { Entry } from "./Entry.js";
import { Vault } from "./Vault.js";
import { EntryID, GroupID } from "../types.js";
/**
 * Group class - contains Entrys
 * @augments VaultItem
 * @memberof module:Buttercup
 */
export declare class Group extends VaultItem {
    static Attribute: Readonly<{
        Role: "bc_group_role";
    }>;
    /**
     * Create a new Group instance within a vault and maybe a group
     * @param vault The vault to add the group to
     * @param parentID Optional parent group ID. If no
     *  value is specified the group is added to the root of the
     *  vault.
     * @returns
     * @memberof Group
     * @static
     */
    static createNew(vault: Vault, parentID?: GroupID, id?: GroupID): Group;
    /**
     * Create a new entry with a title
     * @param title The title of the new entry
     * @returns The new entry
     * @memberof Group
     */
    createEntry(title?: string): Entry;
    /**
     * Create a child group
     * @param title Optionally set a title
     * @returns The new child group
     * @memberof Group
     */
    createGroup(title?: string): Group;
    /**
     * Delete the group
     * If there is a trash group available, the group is moved there. If the group
     * is already in the trash, it is deleted permanently.
     * @param skipTrash Skip the trash
     * @returns True when deleted, false when moved to trash
     * @memberof Group
     */
    delete(skipTrash?: boolean): boolean;
    /**
     * Delete an attribute
     * @param attr The name of the attribute
     * @returns Returns self
     * @memberof Group
     */
    deleteAttribute(attr: string): this;
    /**
     * Find an entry by its ID
     * @param id The ID to search for
     * @returns Null if not found, or the Entry instance
     * @memberof Group
     */
    findEntryByID(id: EntryID): Entry | null;
    /**
     * Find all entries that match a certain property
     * @name findEntriesByProperty
     * @param property The property to search with
     * @param value The value to search for
     * @returns An array of found extries
     * @memberof Group
     */
    findEntriesByProperty(property: RegExp | string, value: RegExp | string): Array<Entry>;
    /**
     * Find a group by its ID
     * @param id The group ID to search for
     * @returns The group or null if not found
     * @memberof Group
     */
    findGroupByID(id: GroupID): Group | null;
    /**
     * Find groups by their title
     * @name findGroupsByTitle
     * @param title The group title
     * @returns An array of groups
     * @memberof Group
     */
    findGroupsByTitle(title: RegExp | string): Array<Group>;
    /**
     * Get an attribute
     * @param attribute The name of the attribute. If none provided
     *  the entire attributes object is returned.
     * @returns Returns the attribute or undefined if not found.
     *  If no attribute name is provided an object containing all attributes is returned.
     * @memberof Group
     */
    getAttribute(attribute?: string): Object | string | undefined;
    /**
     * Get the entries within the group
     * @returns An array of entries
     * @memberof Group
     */
    getEntries(): Array<Entry>;
    /**
     * Get the groups within the group
     * @returns An array of child groups
     * @memberof Group
     */
    getGroups(): Array<Group>;
    /**
     * Get the parent group
     * @returns Returns the parent group instance or null if the parent
     *  is the archive
     * @throws {Error} Throws if no parent could be found (detached)
     * @memberof Group
     */
    getParentGroup(): Group | null;
    /**
     * Get the group title
     * @returns The title of the group
     * @memberof Group
     */
    getTitle(): string;
    /**
     * Check if the group is in the trash
     * @returns Whether or not the group is within the trash group
     * @memberof Group
     */
    isInTrash(): boolean;
    /**
     * Check if the group is used for trash
     * @returns Whether or not the group is the trash group
     * @memberof Group
     */
    isTrash(): boolean;
    /**
     * Move the group to another group or archive
     * @param target The destination Group or Vault instance
     * @returns Self
     * @memberof Group
     */
    moveTo(target: Group | Vault): this;
    /**
     * Set an attribute
     * @param attribute The name of the attribute
     * @param value The value to set
     * @returns Returns self
     * @memberof Group
     */
    setAttribute(attribute: string, value: string): this;
    /**
     * Set the group title
     * @param title The title of the group
     * @returns Returns self
     */
    setTitle(title: string): this;
    _updateRefs(): void;
}
