import EventEmitter from "eventemitter3";
import { ChannelQueue } from "@buttercup/channel-queue";
import { VaultSource } from "./VaultSource.js";
import { StorageInterface } from "../storage/StorageInterface.js";
import { SetTimeout, VaultLiveSnapshot, VaultSourceID } from "../types.js";
export interface InterruptedAutoUpdateFunction {
    (): void | Promise<any>;
}
interface StateChangeEnqueuedFunction {
    (): void | Promise<any>;
}
export interface VaultManagerAddSourceOptions {
    order?: number;
}
export interface VaultManagerOptions {
    autoUpdate?: boolean;
    autoUpdateDelay?: number;
    cacheStorage?: StorageInterface;
    sourceStorage?: StorageInterface;
}
/**
 * Vault manager, to manage vault sources and their vaults
 * @augments EventEmitter
 * @memberof module:Buttercup
 */
export declare class VaultManager extends EventEmitter {
    /**
     * Key prefix for stored vaults
     * @static
     * @memberof VaultManager
     */
    static STORAGE_KEY_PREFIX: string;
    _autoUpdateDelay: number;
    _autoUpdateEnabled: boolean;
    _autoUpdateTimer: SetTimeout;
    _cacheStorage: StorageInterface;
    _initialised: boolean;
    _queue: ChannelQueue;
    _sources: Array<VaultSource>;
    _sourceStorage: StorageInterface;
    /**
     * Construct a new VaultManager
     * @param opts Configuration options
     */
    constructor(opts?: VaultManagerOptions);
    /**
     * Array of vault sources
     * @readonly
     * @memberof VaultManager
     */
    get sources(): Array<VaultSource>;
    /**
     * Array of unlocked vault sources
     * @readonly
     * @memberof VaultManager
     */
    get unlockedSources(): Array<VaultSource>;
    /**
     * @typedef {Object} VaultManagerAddSourceOptions
     * @property {Number=} order Optional order override
     */
    /**
     * Add a VaultSource to the VaultManager
     * - The vault manager will then provide a management
     * platform for the source, including storage access and
     * event aggregation.
     * @param source The source to add
     * @param opts Options for adding the
     *  source
     * @memberof VaultManager
     */
    addSource(source: VaultSource, opts?: VaultManagerAddSourceOptions): Promise<void>;
    /**
     * Dehydrate all sources and write them to storage
     * @returns A promise that resolves once all sources have been dehydrated
     * @memberof VaultManager
     */
    dehydrate(): Promise<void>;
    /**
     * Dehydrate a single vault source
     * @param sourceID The ID of the source
     * @returns A promise that resolves once the source has been dehydrated
     * @memberof VaultManager
     */
    dehydrateSource(sourceOrSourceID: VaultSource | VaultSourceID): Promise<void>;
    /**
     * Enqueue an asychronous change of state
     * @param cb Callback to enqueue
     * @memberof VaultManager
     */
    enqueueStateChange(cb: StateChangeEnqueuedFunction): Promise<void | Promise<any>>;
    /**
     * Fetch all currently available Live Snapshots of vaults
     * @returns An array of snapshot objects
     */
    getLiveSnapshots(): Array<VaultLiveSnapshot>;
    /**
     * Get the next viable order number for a new source
     * @returns The new order
     * @memberof VaultManager
     */
    getNextOrder(): number;
    /**
     * Get a source for an ID
     * @param sourceID The source ID
     * @returns The source with the matching ID or null if not found
     * @memberof VaultManager
     */
    getSourceForID(sourceID: VaultSourceID): VaultSource | null;
    /**
     * Get an array of sources that can be updated
     * @memberof VaultManager
     */
    getUpdateableSources(): Array<VaultSource>;
    /**
     * Initialise the vault manager
     * @memberof VaultManager
     */
    initialise(): void;
    /**
     * Wait for and interrupt state changes when auto-update is running
     * @param cb The callback to execute during the auto-update interruption
     * @returns A promise that resolves when ready
     * @memberof VaultManager
     * @example
     *  vaultManager.interruptAutoUpdate(() => {
     *      // Do something with auto-updating paused
     *  });
     */
    interruptAutoUpdate(cb: InterruptedAutoUpdateFunction): Promise<any>;
    /**
     * Rehydrate sources from storage
     * @returns A promise that resolves once rehydration has completed
     * @memberof VaultManager
     * @throws {Layerr} Rejects if rehydrating from storage fails
     */
    rehydrate(): Promise<void>;
    /**
     * Remove a source from the storage
     * @param sourceID The ID of the source to remove
     * @returns A promise that resolves once the source has been removed
     * @memberof VaultManager
     */
    removeSource(sourceID: VaultSourceID): Promise<void>;
    /**
     * Reorder a source
     * @param sourceID The ID of the source to reorder
     * @param position The 0-based position to move the source to
     * @memberof VaultManager
     * @throws {Layerr} Throws if no source is found
     */
    reorderSource(sourceID: VaultSourceID, position: number): Promise<void>;
    /**
     * Reorder all sources
     * @memberof VaultManager
     */
    reorderSources(): void;
    /**
     * Restore all sources from snapshots that were taken previously
     * @param snapshots An array of snapshot objects
     */
    restoreLiveSnapshots(snapshots: Array<VaultLiveSnapshot>): Promise<void>;
    /**
     * Toggle auto updating of sources
     * @param enable Enable or disable auto updating. Leave empty
     *  to invert the setting
     * @param delay Milliseconds between updates
     * @memberof VaultManager
     */
    toggleAutoUpdating(enable?: boolean, delay?: number): void;
    _autoUpdateSources(): Promise<void | Promise<any>>;
    _migrateLegacyVaults(): Promise<void>;
    _startAutoUpdateTimer(): Promise<void>;
    _stopAutoUpdateTimer(): Promise<void>;
    _storeDehydratedSource(id: VaultSourceID, dehydratedSource: string): Promise<void>;
}
export {};
