import EventEmitter from "eventemitter3";
import { ChannelQueue } from "@buttercup/channel-queue";
import { Vault } from "./Vault.js";
import { Credentials } from "../credentials/Credentials.js";
import { AttachmentManager } from "../attachments/AttachmentManager.js";
import { TextDatasource } from "../datasources/TextDatasource.js";
import { VaultManager } from "./VaultManager.js";
import { VaultFormatID, VaultLiveSnapshot, VaultSourceID, VaultSourceStatus } from "../types.js";
interface StateChangeEnqueuedFunction {
    (): void | Promise<any>;
}
export interface VaultSourceConfig {
    colour?: string;
    id?: VaultSourceID;
    order?: number;
    meta?: VaultSourceMetadata;
}
export interface VaultSourceSaveOptions {
    storeOfflineCopy?: boolean;
}
export interface VaultSourceUnlockOptions {
    initialiseRemote?: boolean;
    loadOfflineCopy?: boolean;
    storeOfflineCopy?: boolean;
}
export interface VaultSourceMetadata {
    [property: string]: any;
}
/**
 * Vault source class for managing a single vault
 * within a vault manager
 * @augments EventEmitter
 * @memberof module:Buttercup
 */
export declare class VaultSource extends EventEmitter {
    static STATUS_LOCKED: VaultSourceStatus;
    static STATUS_PENDING: VaultSourceStatus;
    static STATUS_UNLOCKED: VaultSourceStatus;
    /**
     * Rehydrate the vault source from a dehydrated state
     * @param dehydratedString The dehydrated form of the vault source
     * @returns A rehydrated instance
     * @memberof VaultSource
     * @static
     */
    static rehydrate(dehydratedString: string): VaultSource;
    _attachmentManager: AttachmentManager;
    _colour: string;
    _credentials: string | Credentials;
    _datasource: TextDatasource;
    _id: VaultSourceID;
    _meta: VaultSourceMetadata;
    _name: string;
    _order: number;
    _queue: ChannelQueue;
    _shares: Array<any>;
    _status: VaultSourceStatus;
    _type: string;
    _vault: Vault;
    _vaultManager: VaultManager;
    constructor(name: string, type: string, credentialsString: string, config?: VaultSourceConfig);
    /**
     * The attachment manager
     * @memberof VaultSource
     * @readonly
     */
    get attachmentManager(): AttachmentManager;
    /**
     * Source colour
     * @memberof VaultSource
     */
    get colour(): string;
    /**
     * Source ID
     * @memberof VaultSource
     * @readonly
     */
    get id(): VaultSourceID;
    /**
     * Meta data
     * @memberof VaultSource
     * @readonly
     */
    get meta(): VaultSourceMetadata;
    /**
     * Source name
     * @memberof VaultSource
     * @readonly
     */
    get name(): string;
    /**
     * The vault order on a vault management instance
     * @memberof VaultSource
     * @readonly
     */
    get order(): number;
    /**
     * Source status
     * @memberof VaultSource
     * @readonly
     */
    get status(): VaultSourceStatus;
    /**
     * The datasource type
     * @memberof VaultSource
     * @readonly
     */
    get type(): string;
    /**
     * Vault reference
     * @memberof VaultSource
     * @readonly
     */
    get vault(): Vault;
    set colour(newColour: string);
    set order(newOrder: number);
    /**
     * Change the master vault password
     * @param oldPassword The original/current password
     * @param newPassword The new password to change to
     * @param meta Optional metadata
     * @memberof VaultSource
     */
    changeMasterPassword(oldPassword: string, newPassword: string, meta?: {
        [key: string]: any;
    }): Promise<void>;
    /**
     * Check if the vault source can be updated
     * @memberof VaultSource
     */
    canBeUpdated(): boolean;
    /**
     * Check if the source has an offline copy
     * @returns A promise which resolves with whether an offline
     *  copy is available or not
     * @memberof VaultSource
     */
    checkOfflineCopy(): Promise<boolean>;
    /**
     * Convert vault to a new format
     * @param targetFormat The target format to convert to
     * @memberof VaultSource
     */
    convert(targetFormat: VaultFormatID): Promise<void>;
    /**
     * Dehydrate the source to a JSON string, ready for storage
     * @memberof VaultSource
     */
    dehydrate(): Promise<string>;
    /**
     * Get a live snapshot of the current unlocked state
     * @returns A snapshot object
     */
    getLiveSnapshot(): VaultLiveSnapshot;
    /**
     * Get offline content, if it exists
     * @returns A promise a resolves with the content, or null
     *  if it doesn't exist
     * @memberof VaultSource
     */
    getOfflineContent(): Promise<string | null>;
    /**
     * Detect whether the local archives (in memory) differ from their remote copies
     * Fetches the remote copies from their datasources and detects differences between
     * them and their local counterparts. Does not change/update the local items.
     * @returns A promise that resolves with a boolean - true if
     *      there are differences, false if there is not
     * @memberof VaultSource
     */
    localDiffersFromRemote(): Promise<boolean>;
    /**
     * Lock the source
     * @memberof VaultSource
     */
    lock(): Promise<void>;
    /**
     * Merge remote contents
     * Detects differences between a local and a remote item, and merges the
     * two copies together.
     * @returns A promise that resolves with the newly merged archive -
     *      This archive is automatically saved over the original local copy.
     * @memberof VaultSource
     */
    mergeFromRemote(): Promise<Vault>;
    /**
     * Rename the vault source
     * @param name The new name
     * @memberof VaultSource
     */
    rename(name: string): void;
    /**
     * Restore unlocked state from a live snapshot
     * @param snapshot The snapshot taken previously
     */
    restoreFromLiveSnapshot(snapshot: VaultLiveSnapshot): Promise<void>;
    /**
     * Save the vault to the remote, ensuring that it's first merged and
     * updated to prevent conflicts or overwrites.
     * @memberof VaultSource
     */
    save(config?: VaultSourceSaveOptions): Promise<void>;
    supportsAttachments(): boolean;
    testMasterPassword(password: string): Promise<boolean>;
    unlock(vaultCredentials: Credentials, config?: VaultSourceUnlockOptions): Promise<void>;
    /**
     * Update the vault
     * @returns A promise that resolves once the update has
     *  completed
     * @memberof VaultSource
     */
    update({ skipDiff }?: {
        skipDiff?: boolean;
    }): Promise<void>;
    /**
     * Write the vault to the remote
     * - This does not perform any merging or sync checks, but simply
     * writes the vault contents to the remote, overwriting whatever
     * was there before.
     * @returns A promise that resolves when saving has completed
     * @memberof VaultSource
     */
    write(): Promise<void>;
    _applyShares(): void;
    _enqueueStateChange(cb: StateChangeEnqueuedFunction, stack?: string): Promise<any>;
    _unloadShares(): void;
    _updateCredentialsFromDatasource(): Promise<void>;
    _updateInsights(): Promise<void>;
    _updateVaultCredentials(newCredentials: any): Promise<void>;
    _waitNonPending(): Promise<void>;
}
export {};
