import { CredentialsData, CredentialsPayload, DatasourceConfiguration } from "../types.js";
/**
 * Secure credentials storage/transfer class
 * - Allows for the safe transfer of credentials within the
 * Buttercup application environment. Will not allow
 * credentials to be shared or transferred outside of the
 * environment. Credential properties are stored in memory
 * and are inaccessible to public functions.
 */
export declare class Credentials {
    static PURPOSE_DECRYPT_VAULT: string;
    static PURPOSE_ENCRYPT_VAULT: string;
    static PURPOSE_SECURE_EXPORT: string;
    /**
     * Get all available purposes
     * @static
     */
    static allPurposes(): string[];
    /**
     * Create a new Credentials instance using an existing Credentials
     * instance - This can be used to reset a credentials's purposes.
     * @param credentials A credentials instance
     * @param masterPassword The master password used to
     *  encrypt the instance being cloned
     * @throws {Error} Throws if no master password provided
     * @throws {Error} Throws if master password does not match
     *  original
     */
    static fromCredentials(credentials: Credentials, masterPassword: string): Credentials;
    /**
     * Create a new Credentials instance from a Datasource configuration
     * @param datasourceConfig The configuration for the
     *  datasource - this usually includes the credential data used for
     *  authenticating against the datasource host platform.
     * @param masterPassword Optional master password to
     *  store alongside the credentials. Used to create secure strings.
     */
    static fromDatasource(datasourceConfig: DatasourceConfiguration, masterPassword?: string): Credentials;
    /**
     * Create a new Credentials instance from a password
     * - uses the single password value as the master password stored
     * alongside the original password if no master password is
     * provided. The master password is used when generating secure
     * strings.
     * @param password The password to store
     * @param masterPassword Optional master password
     *  to store alongside the credentials. Used to create secure
     *  strings.
     */
    static fromPassword(password: string, masterPassword?: string): Credentials;
    /**
     * Create a new instance from a secure string
     * @param content Encrypted content
     * @param masterPassword The password for decryption
     * @returns A promise that resolves with the new instance
     */
    static fromSecureString(content: string, masterPassword: string): Promise<Credentials>;
    /**
     * Check if a value is an instance of Credentials
     * @param inst The value to check
     */
    static isCredentials(inst: Credentials | any): boolean;
    id: string;
    /**
     * Create a new Credentials instance
     * @param obj Object data representing some credentials
     * @param masterPassword Optional master password to store with
     *  the credentials data, which is used for generating secure strings.
     */
    constructor(obj?: CredentialsData, masterPassword?: string);
    /**
     * Get raw credentials data (only available in specialised environments)
     * @returns Credentials data object, or null if not available
     */
    getCredentialsData(): CredentialsData | null;
    /**
     * Get raw credentials data (only available in specialised environments)
     */
    getData(): CredentialsPayload | null;
    /**
     * Restrict the purposes that this set of credentials
     * can be used for. Once a purpose is removed it can
     * no longer be added again to the same instance.
     * @param allowedPurposes An array of
     *  new allowed purposes. If a purpose mentioned is
     *  not currently permitted, it will be ignored.
     * @returns Returns self
     * @example
     *  credentials.restrictPurposes(
     *      Credentials.PURPOSE_SECURE_EXPORT
     *  );
     *  // credentials can only be exported to an
     *  // encrypted string, and not used for things
     *  // like encrypting datasource changes.
     */
    restrictPurposes(allowedPurposes: Array<string>): this;
    /**
     * Set the credential data
     * @param data The credentials data to overwrite the old with
     */
    setCredentialsData(data: CredentialsData): void;
    /**
     * Convert the credentials to an encrypted string, for storage
     * @returns A promise that resolves with the encrypted credentials
     * @throws {Error} Rejects when masterPassword is not a string
     * @throws {Error} Rejects if credentials don't permit secure export purposes
     */
    toSecureString(): Promise<string>;
    /**
     * Get raw credentials data (only available in specialised environments)
     * @protected
     * @deprecated
     */
    _getData(): CredentialsPayload;
}
