import { TextDatasource } from "./TextDatasource.js";
import { Credentials } from "../credentials/Credentials.js";
import { AttachmentDetails, BufferLike, DatasourceConfigurationFile, DatasourceLoadedData, EncryptedContent, History, VaultID } from "../types.js";
/**
 * File datasource for loading and saving files
 * @augments TextDatasource
 * @memberof module:Buttercup
 */
export declare class FileDatasource extends TextDatasource {
    _filename: string;
    mkdir: Function;
    readFile: Function;
    stat: Function;
    unlink: Function;
    writeFile: Function;
    /**
     * Constructor for the file datasource
     * @param credentials The credentials instance with which to
     *  use to configure the datasource
     */
    constructor(credentials: Credentials);
    get baseDir(): string;
    /**
     * The file path
     * @memberof FileDatasource
     */
    get path(): string;
    /**
     * Ensure attachment paths exist
     * @memberof FileDatasource
     * @protected
     */
    _ensureAttachmentsPaths(vaultID: VaultID): Promise<void>;
    /**
     * Get encrypted attachment
     * - Loads the attachment contents from a file into a buffer
     * @param vaultID The ID of the vault
     * @param attachmentID The ID of the attachment
     * @memberof FileDatasource
     */
    getAttachment(vaultID: VaultID, attachmentID: string): Promise<BufferLike>;
    /**
     * Get the datasource configuration
     * @memberof FileDatasource
     */
    getConfiguration(): DatasourceConfigurationFile;
    /**
     * Load from the filename specified in the constructor using a password
     * @param credentials The credentials for decryption
     * @returns A promise resolving with archive history
     * @memberof FileDatasource
     */
    load(credentials: Credentials): Promise<DatasourceLoadedData>;
    /**
     * Put attachment data
     * @param vaultID The ID of the vault
     * @param attachmentID The ID of the attachment
     * @param buffer The attachment data
     * @param details The attachment details
     * @memberof FileDatasource
     */
    putAttachment(vaultID: VaultID, attachmentID: string, buffer: BufferLike, details: AttachmentDetails): Promise<void>;
    /**
     * Remove an attachment
     * @param vaultID The ID of the vault
     * @param attachmentID The ID of the attachment
     * @memberof FileDatasource
     */
    removeAttachment(vaultID: VaultID, attachmentID: string): Promise<void>;
    /**
     * Save archive history to a file
     * @param history The archive history to save
     * @param credentials The credentials to save with
     * @returns A promise that resolves when saving is complete
     * @memberof FileDatasource
     */
    save(history: History, credentials: Credentials): Promise<EncryptedContent>;
    /**
     * Whether or not the datasource supports attachments
     * @memberof FileDatasource
     */
    supportsAttachments(): boolean;
    /**
     * Whether or not the datasource supports bypassing remote fetch operations
     * @returns True if content can be set to bypass fetch operations,
     *  false otherwise
     * @memberof FileDatasource
     */
    supportsRemoteBypass(): boolean;
}
